import pytest
from rest_framework import exceptions
from rest_framework.pagination import PAGE_BREAK, PageLink
from rest_framework.request import Request
from rest_framework.test import APIRequestFactory

import drf_link_header_pagination

factory = APIRequestFactory()


class TestLinkHeaderPagination:
    """
    Unit tests for `pagination.LinkHeaderPagination`.
    """

    def setup(self):
        class ExamplePagination(drf_link_header_pagination.LinkHeaderPagination):
            page_size = 5

        self.pagination = ExamplePagination()
        self.queryset = range(1, 101)

    def paginate_queryset(self, request):
        return list(self.pagination.paginate_queryset(self.queryset, request))

    def get_paginated_response(self, queryset):
        return self.pagination.get_paginated_response(queryset)

    def get_html_context(self):
        return self.pagination.get_html_context()

    def test_no_page_number(self):
        request = Request(factory.get("/"))
        queryset = self.paginate_queryset(request)
        response = self.get_paginated_response(queryset)
        context = self.get_html_context()
        assert queryset == [1, 2, 3, 4, 5]
        assert response.data == [1, 2, 3, 4, 5]
        assert response["Link"] == (
            '<http://testserver/?page=2>; rel="next", '
            '<http://testserver/?page=20>; rel="last"'
        )
        assert context == {
            "previous_url": None,
            "next_url": "http://testserver/?page=2",
            "page_links": [
                PageLink("http://testserver/", 1, True, False),
                PageLink("http://testserver/?page=2", 2, False, False),
                PageLink("http://testserver/?page=3", 3, False, False),
                PAGE_BREAK,
                PageLink("http://testserver/?page=20", 20, False, False),
            ],
        }
        assert self.pagination.display_page_controls
        assert isinstance(self.pagination.to_html(), type(""))

    def test_second_page(self):
        request = Request(factory.get("/", {"page": 2}))
        queryset = self.paginate_queryset(request)
        response = self.get_paginated_response(queryset)
        context = self.get_html_context()
        assert queryset == [6, 7, 8, 9, 10]
        assert response.data == [6, 7, 8, 9, 10]
        assert response["Link"] == (
            '<http://testserver/>; rel="prev", '
            '<http://testserver/?page=3>; rel="next", '
            '<http://testserver/>; rel="first", '
            '<http://testserver/?page=20>; rel="last"'
        )
        assert context == {
            "previous_url": "http://testserver/",
            "next_url": "http://testserver/?page=3",
            "page_links": [
                PageLink("http://testserver/", 1, False, False),
                PageLink("http://testserver/?page=2", 2, True, False),
                PageLink("http://testserver/?page=3", 3, False, False),
                PAGE_BREAK,
                PageLink("http://testserver/?page=20", 20, False, False),
            ],
        }

    def test_last_page(self):
        request = Request(factory.get("/", {"page": "last"}))
        queryset = self.paginate_queryset(request)
        response = self.get_paginated_response(queryset)
        context = self.get_html_context()
        assert queryset == [96, 97, 98, 99, 100]
        assert response.data == [96, 97, 98, 99, 100]
        assert response["Link"] == (
            '<http://testserver/?page=19>; rel="prev", '
            '<http://testserver/>; rel="first"'
        )
        assert context == {
            "previous_url": "http://testserver/?page=19",
            "next_url": None,
            "page_links": [
                PageLink("http://testserver/", 1, False, False),
                PAGE_BREAK,
                PageLink("http://testserver/?page=18", 18, False, False),
                PageLink("http://testserver/?page=19", 19, False, False),
                PageLink("http://testserver/?page=20", 20, True, False),
            ],
        }

    def test_invalid_page(self):
        request = Request(factory.get("/", {"page": "invalid"}))
        with pytest.raises(exceptions.NotFound):
            self.paginate_queryset(request)



import pytest
from rest_framework import exceptions
from rest_framework.request import Request
from rest_framework.test import APIRequestFactory

import drf_link_header_pagination

from .mocks import MockObject, MockQuerySet

factory = APIRequestFactory()


class TestLinkHeaderCursorPagination:
    """
    Unit tests for `pagination.LinkHeaderCursorPagination`.

    Tests inspired from:
        https://github.com/encode/django-rest-framework/blob/master/tests/test_pagination.py
    """

    def setup(self):
        class ExamplePagination(drf_link_header_pagination.LinkHeaderCursorPagination):
            page_size = 5
            page_size_query_param = "page_size"
            max_page_size = 20
            ordering = "created"

        self.pagination = ExamplePagination()
        self.queryset = MockQuerySet(
            [MockObject(idx) for idx in [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]]
        )

    def get_page_urls(self, url):
        """
        Given a URL return a tuple of:
        (previous url, next url)
        """
        request = Request(factory.get(url))
        self.pagination.paginate_queryset(self.queryset, request)

        next_url = self.pagination.get_next_link()
        previous_url = self.pagination.get_previous_link()

        return (previous_url, next_url)

    def paginate_queryset(self, request):
        return list(self.pagination.paginate_queryset(self.queryset, request))

    def get_paginated_response(self, queryset):
        return self.pagination.get_paginated_response(queryset)

    def get_html_context(self):
        return self.pagination.get_html_context()

    def test_no_cursor(self):
        url = "/"
        request = Request(factory.get(url))
        queryset = self.paginate_queryset(request)
        response = self.get_paginated_response(queryset)
        context = self.get_html_context()
        queryset_values = [item.value for item in queryset]
        response_data_values = [item.value for item in response.data]
        (previous_url, next_url) = self.get_page_urls(url)
        assert queryset_values == [1, 2, 3, 4, 5]
        assert response_data_values == [1, 2, 3, 4, 5]
        assert response["Link"] == ('<{}>; rel="next"'.format(next_url))
        assert context == {
            "previous_url": None,
            "next_url": next_url,
        }
        assert self.pagination.display_page_controls
        assert isinstance(self.pagination.to_html(), str)

    def test_second_page(self):
        first_page_url = "/"
        (_, second_page_url) = self.get_page_urls(first_page_url)

        request = Request(factory.get(second_page_url))
        queryset = self.paginate_queryset(request)
        response = self.get_paginated_response(queryset)
        context = self.get_html_context()
        queryset_values = [item.value for item in queryset]
        response_data_values = [item.value for item in response.data]
        (previous_url, next_url) = self.get_page_urls(second_page_url)
        assert queryset_values == [6, 7, 8, 9, 10]
        assert response_data_values == [6, 7, 8, 9, 10]
        assert response["Link"] == ('<{}>; rel="prev", <{}>; rel="next"').format(
            previous_url, next_url
        )
        assert context == {
            "previous_url": previous_url,
            "next_url": next_url,
        }

    def test_invalid_page(self):
        request = Request(factory.get("/", {"cursor": "invalid"}))
        with pytest.raises(exceptions.NotFound):
            self.paginate_queryset(request)



from rest_framework.request import Request
from rest_framework.test import APIRequestFactory

import drf_link_header_pagination

from .mocks import MockObject, MockQuerySet
from .test_link_header_cursor_pagination import TestLinkHeaderCursorPagination

factory = APIRequestFactory()


class TestLinkHeaderLinkResponseCursorPagination(TestLinkHeaderCursorPagination):
    """
    Unit tests for `pagination.LinkHeaderLinkResponseCursorPagination`.
    """

    def setup(self):
        class ExamplePagination(
            drf_link_header_pagination.LinkHeaderLinkResponseCursorPagination
        ):
            page_size = 5
            page_size_query_param = "page_size"
            max_page_size = 20
            ordering = "created"

        self.pagination = ExamplePagination()
        self.queryset = MockQuerySet(
            [MockObject(idx) for idx in [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]]
        )

    def test_no_cursor(self):
        url = "/"
        request = Request(factory.get(url))
        queryset = self.paginate_queryset(request)
        response = self.get_paginated_response(queryset)
        response_next_url = response.data["next"]
        response_previous_url = response.data["previous"]
        (previous_url, next_url) = self.get_page_urls(url)
        assert response_next_url == next_url
        assert response_previous_url == previous_url
        response_data_values = [item.value for item in response.data["results"]]
        assert response_data_values == [1, 2, 3, 4, 5]

    def test_second_page(self):
        first_page_url = "/"
        (_, second_page_url) = self.get_page_urls(first_page_url)

        request = Request(factory.get(second_page_url))
        queryset = self.paginate_queryset(request)
        response = self.get_paginated_response(queryset)
        response_next_url = response.data["next"]
        response_previous_url = response.data["previous"]
        (previous_url, next_url) = self.get_page_urls(second_page_url)
        assert response_next_url == next_url
        assert response_previous_url == previous_url
        response_data_values = [item.value for item in response.data["results"]]
        assert response_data_values == [6, 7, 8, 9, 10]



class MockObject:
    def __init__(self, idx):
        self.created = idx
        self.value = idx


class MockQuerySet:
    def __init__(self, items):
        self.items = items

    def filter(self, created__gt=None, created__lt=None):
        if created__gt is not None:
            return MockQuerySet(
                [item for item in self.items if item.created > int(created__gt)]
            )

        assert created__lt is not None
        return MockQuerySet(
            [item for item in self.items if item.created < int(created__lt)]
        )

    def order_by(self, *ordering):
        if ordering[0].startswith("-"):
            return MockQuerySet(list(reversed(self.items)))
        return self

    def __getitem__(self, sliced):
        return self.items[sliced]



from rest_framework.pagination import CursorPagination, PageNumberPagination
from rest_framework.response import Response
from rest_framework.utils.urls import remove_query_param, replace_query_param

__all__ = [
    "LinkHeaderPagination",
    "LinkHeaderCursorPagination",
    "LinkHeaderLinkResponseCursorPagination",
]


class LinkHeaderMixin:
    def get_headers(self):
        """Prepare and return link headers."""
        links = []
        for label, method_name in (
            ("prev", "get_previous_link"),
            ("next", "get_next_link"),
            ("first", "get_first_link"),
            ("last", "get_last_link"),
        ):
            try:
                method = getattr(self, method_name)
            except AttributeError:
                continue
            links.append((method(), label))

        header_links = []
        for url, label in links:
            if url is not None:
                header_links.append('<{}>; rel="{}"'.format(url, label))

        return {"Link": ", ".join(header_links)} if header_links else {}

    def get_paginated_response(self, data):
        return Response(data, headers=self.get_headers())


class LinkHeaderPagination(LinkHeaderMixin, PageNumberPagination):
    """Inform the user of pagination links via response headers, similar to
    what's described in
    https://developer.github.com/guides/traversing-with-pagination/.
    """

    def get_first_link(self):
        if not self.page.has_previous():
            return None

        url = self.request.build_absolute_uri()

        return remove_query_param(url, self.page_query_param)

    def get_last_link(self):
        if not self.page.has_next():
            return None

        url = self.request.build_absolute_uri()

        return replace_query_param(
            url, self.page_query_param, self.page.paginator.num_pages
        )

    def get_paginated_response_schema(self, schema):
        return schema


class LinkHeaderCursorPagination(LinkHeaderMixin, CursorPagination):
    """
    Customized cursor pagination with links provided via:
        - headers.
    """

    def get_paginated_response_schema(self, schema):
        return schema


class LinkHeaderLinkResponseCursorPagination(LinkHeaderMixin, CursorPagination):
    """
    Customized cursor pagination with links provided via:
        - content of the response
        - headers.
    """

    def get_paginated_response(self, data):
        return super().get_paginated_response(
            {
                "next": self.get_next_link(),
                "previous": self.get_previous_link(),
                "results": data,
            }
        )
